<?php
/**
 * Main application class
 *
 * @package wp-email-countdown
 * @author InstaProduct
 * @link http://insta-product.com/
 */

//
//Include all function files
require_once( IPCOZR_LIB . '/config.php' );
require_once( IPCOZR_LIB . '/setting.php' );
require_once( IPCOZR_LIB . '/db.php' );
require_once( IPCOZR_LIB . '/model.php' );
require_once( IPCOZR_LIB . '/modelWidget.php' );
require_once( IPCOZR_LIB . '/modelPromo.php' );
require_once( IPCOZR_LIB . '/modelStat.php' );
require_once( IPCOZR_APP . '/controller.php' );
require_once( IPCOZR_APP . '/viewer.php' );

if ( ! class_exists( 'IPCozr_App' ))
{
    class IPCozr_App
    {
        //{{ run

        /**
         * Run all functions
         *
         * @return void
         */
        public static function run()
        {
            $hook = sanitize_title( IPCOZR_TITLE );

            //Install Plugin
            register_activation_hook( __FILE__, array( __CLASS__, 'install' ) );

            //Hook for plugin loaded
            add_action( 'plugins_loaded', array( __CLASS__, 'updateDb' ) );

            //
            //Admin init
            add_action( 'admin_init', array( __CLASS__, 'adminInit' ) );

            //
            //Register admin menu
            add_action( 'admin_menu', array( __CLASS__, 'addAdminMenu' ) );

            /*
             * Rewrite Handler
             */
            add_action( 'init', array( __CLASS__, 'rewriteFlush' ) );
            add_action( 'pre_get_posts', array( __CLASS__, 'rewriteInit' ) );

            //Render page
            add_action( 'template_redirect', array( __CLASS__, 'renderPage' ) );

            /**
             * Shortcode setup
             *
             */
            // Add generator shortcode button
            add_action( 'init', array( __CLASS__, 'addShortcodeEditorButton' ), 100 );
            // Add pop up content
            add_filter( 'the_editor', array( __CLASS__, 'shortcodeEditorFilter' ) );
            add_action( 'admin_footer', array( __CLASS__, 'addShortcodeForm' ) );
            //Add Shortcode
            add_shortcode( 'campaign', array( __CLASS__, 'shortcode' ) );

            /*
             * Add meta box option
             */
            add_action( 'add_meta_boxes', array( __CLASS__, 'addMetaBox' ) );
            add_action( 'save_post', array( __CLASS__, 'savePost' ) );

            //Add widget via meta box
            add_action( 'wp_footer', array( __CLASS__, 'widgetPage' ) );

            /*
             * Register Widget
             */
            add_action( 'widgets_init', array( __CLASS__, 'widgetInit' ));

            //
            //Add css
            add_action( 'admin_print_styles-toplevel_page_' . IPCOZR_PAGENAME, array( __CLASS__, 'adminStyle' ), 1100 );
            add_action( 'admin_print_styles-'.$hook.'_page_' . IPCOZR_PAGENAME . '-addnew', array( __CLASS__, 'adminStyle' ), 1100 );
            add_action( 'admin_print_styles-'.$hook.'_page_' . IPCOZR_PAGENAME . '-widget', array( __CLASS__, 'adminStyle' ), 1100 );
            add_action( 'admin_print_styles-'.$hook.'_page_' . IPCOZR_PAGENAME . '-promo', array( __CLASS__, 'adminStyle' ), 1100 );
            add_action( 'admin_print_styles-'.$hook.'_page_' . IPCOZR_PAGENAME . '-setting', array( __CLASS__, 'adminStyle' ), 1100 );
            add_action( 'admin_print_styles', array( __CLASS__, 'adminGlobalStyle' ), 1000 );

            //
            //Add javascript
            add_action( 'admin_print_scripts-toplevel_page_' . IPCOZR_PAGENAME, array( __CLASS__, 'adminScript' ), 1100 );
            add_action( 'admin_print_scripts-'.$hook.'_page_' . IPCOZR_PAGENAME . '-addnew', array( __CLASS__, 'adminScript' ), 1100 );
            add_action( 'admin_print_scripts-'.$hook.'_page_' . IPCOZR_PAGENAME . '-widget', array( __CLASS__, 'adminScript' ), 1100 );
            add_action( 'admin_print_scripts-'.$hook.'_page_' . IPCOZR_PAGENAME . '-promo', array( __CLASS__, 'adminScript' ), 1100 );
            add_action( 'admin_print_scripts-'.$hook.'_page_' . IPCOZR_PAGENAME . '-setting', array( __CLASS__, 'adminScript' ), 1100 );
            add_action( 'admin_print_scripts', array( __CLASS__, 'adminGlobalScript' ), 1000 );

            //Front style & script
            add_action('wp_head', array( __CLASS__, 'publicStyle' ) );

            if ( ! is_admin() ) {
                add_action('wp_enqueue_scripts', array( __CLASS__, 'publicScript' ), 10 );
            }
        }

        //}}
        //{{ install

        /**
         * Install plugin
         *
         * @return void
         */
        public static function install()
        {
            //Install DB
            $db = new IPCozr_Db();
            $db->install();
        }

        //}}
        //{{ updateDb

        /**
         * Update Db
         *
         * @return void
         */
        public static function updateDb()
        {
            $db = new IPCozr_Db();
            $db->update();
        }

        //}}
        //{{ addAdminMenu

        /**
         * Add admin menu
         *
         * @return void
         */
        public static function addAdminMenu()
        {
            add_menu_page(
                IPCOZR_TITLE,
                IPCOZR_TITLE,
                'edit_posts',
                IPCOZR_PAGENAME,
                array( __CLASS__, 'dashboard' ),
                'dashicons-backup'
            );
            add_submenu_page(
                IPCOZR_PAGENAME,
                __( 'All Campaigns', IPCOZR_SLUG ),
                __( 'All Campaigns', IPCOZR_SLUG ),
                'edit_posts',
                IPCOZR_PAGENAME,
                array( __CLASS__, 'dashboard' )
            );
            add_submenu_page(
                IPCOZR_PAGENAME,
                __( 'Add Campaign', IPCOZR_SLUG ),
                __( 'Add Campaign', IPCOZR_SLUG ),
                'edit_posts',
                IPCOZR_PAGENAME . '-addnew',
                array( __CLASS__, 'addnew' )
            );
            add_submenu_page(
                IPCOZR_PAGENAME,
                __( 'Widget', IPCOZR_SLUG ),
                __( 'Widget', IPCOZR_SLUG ),
                'edit_posts',
                IPCOZR_PAGENAME . '-widget',
                array( __CLASS__, 'widget' )
            );
            add_submenu_page(
                IPCOZR_PAGENAME,
                __( 'Promo Page', IPCOZR_SLUG ),
                __( 'Promo Page', IPCOZR_SLUG ),
                'edit_posts',
                IPCOZR_PAGENAME . '-promo',
                array( __CLASS__, 'promo' )
            );
            add_submenu_page(
                IPCOZR_PAGENAME,
                __( 'Settings', IPCOZR_SLUG ),
                __( 'Settings', IPCOZR_SLUG ),
                'edit_posts',
                IPCOZR_PAGENAME . '-setting',
                array( __CLASS__, 'setting' )
            );
        }

        //}}
        //{{ adminInit

        /**
         * Admin init
         *
         * @return void
         */
        public static function adminInit()
        {
            /**
             * Ajax handler
             **/
            $controller = new IPCozr_Controller();
            $controller->ajax();
        }

        //}}
        //{{ dashboard

        /**
         * Render admin dashboard page
         *
         * @return void
         */
        public static function dashboard()
        {
            $viewer = new IPCozr_Viewer();
            if ( ! empty( $_GET['action'] ) && $_GET['action'] == 'edit' ) {
                $viewer->edit();
            } else {
                $viewer->dashboard();
            }
        }

        //}}
        //{{ addnew

        /**
         * Render add new page
         *
         * @return void
         */
        public static function addnew()
        {
            $viewer = new IPCozr_Viewer();
            $viewer->addnew();
        }

        //}}
        //{{ widget

        /**
         * Render manage widget
         *
         * @return void
         */
        public static function widget()
        {
            $viewer = new IPCozr_Viewer();
            if ( ! empty( $_GET['action'] ) && $_GET['action'] == 'edit' ) {
                $viewer->widgetEdit();
            } elseif ( ! empty( $_GET['action'] ) && $_GET['action'] == 'create' ) {
                $viewer->widgetCreate();
            } else {
                $viewer->widget();
            }
        }

        //}}
        //{{ promo

        /**
         * Render manage promo
         *
         * @return void
         */
        public static function promo()
        {
            $viewer = new IPCozr_Viewer();
            if ( ! empty( $_GET['action'] ) && $_GET['action'] == 'edit' ) {
                $viewer->promoEdit();
            } elseif ( ! empty( $_GET['action'] ) && $_GET['action'] == 'create' ) {
                $viewer->promoCreate();
            } else {
                $viewer->promo();
            }
        }

        //}}
        //{{ setting

        /**
         * Render manage setting
         *
         * @return void
         */
        public static function setting()
        {
            $viewer = new IPCozr_Viewer();
            $viewer->setting();
        }

        //}}
        //{{ rewriteInit

        /**
         * Init request
         *
         * @return void
         */
        public static function rewriteInit( $query )
        {
            $q = $query->query;
            if ( isset( $q['ipcozr'] ) && isset( $q['promo_name'] )) {
                $query->query_vars['post_type'] = 'ipcozr-promo';
                $query->query_vars['name']      = $q['promo_name'];
                $query->query_vars['pagename']  = $q['promo_name'];
                $query->query['pagename']       = $q['promo_name'];

            } elseif( isset( $q['promo_id'] )) {
                $query->query_vars['post_type'] = 'ipcozr-promo';
                $query->query_vars['p']         = absint( $q['promo_id'] );

            } elseif( isset( $q['ipcozr'] ) && isset( $q['ipcozr_page'] ) && ! empty( $q['campaign_id'] ) ) {
                $query->query_vars['post_type'] = 'ipcozr-item';
                $query->query_vars['p']         = absint( $q['campaign_id'] );
            }
        }

        //}}
        //{{ registerRewrite

        /**
         * Init rewrite rule
         *
         * @return void
         */
        public static function registerRewrite()
        {
            $base = get_option( '__ipcozr_promo_base_path', 'promo' );
            add_rewrite_rule(
                $base . '/([^/]+)/?$',
                'index.php?ipcozr=1&promo_name=$matches[1]',
                'top'
            );
            add_rewrite_rule(
                $base . '/([^/]+)/feed/(feed|rdf|rss|rss2|atom)/?$',
                'index.php?ipcozr=1&promo_name=$matches[1]&feed=$matches[2]',
                'top'
            );
            add_rewrite_rule(
                $base . '/([^/]+)/(feed|rdf|rss|rss2|atom)/?$',
                'index.php?ipcozr=1&promo_name=$matches[1]&feed=$matches[2]',
                'top'
            );
            add_rewrite_rule(
                $base . '/([^/]+)/page/?([0-9]{1,})/?$',
                'index.php?ipcozr=1&promo_name=$matches[1]&paged=$matches[2]',
                'top'
            );
            add_rewrite_rule(
                'wp-email-countdown-campaign/?$',
                'index.php?ipcozr=1&ipcozr_page=campaign',
                'top'
            );
            add_rewrite_rule(
                'wp-email-countdown-image/?$',
                'index.php?ipcozr=1&ipcozr_page=image',
                'top'
            );
            add_rewrite_tag( '%promo_name%', '([a-zA-Z0-9\_])');
            add_rewrite_tag( '%promo_id%', '([0-9])');
            add_rewrite_tag( '%campaign_id%', '([0-9])');
            add_rewrite_tag( '%campaign_preview%', '([0-9])');
            add_rewrite_tag( '%ipcozr_page%', '([0-9])');
            add_rewrite_tag( '%ipcozr%', '([1])');
        }

        //}}
        //{{ rewriteFlush

        /**
         * Flush rewrite rule
         *
         * @return void
         */
        public static function rewriteFlush()
        {
            $base = get_option( '__ipcozr_promo_base_path', 'promo' );
            $old  = get_option( '__ipcozr_promo_base_path_old', 'promo' );

            self::registerRewrite();

            if ( $base != $old ) {
                flush_rewrite_rules();
                update_option( '__ipcozr_promo_base_path_old', $base );
            }
        }

        //}}
        //{{ renderPage

        /**
         * Render promo page
         *
         * @return void
         */
        public static function renderPage()
        {
            //Render view
            $viewer = new IPCozr_Viewer();
            $viewer->page();
        }

        //}}
        //{{ addShortcodeEditorButton

        /**
         * Add shortcode button in content editor
         *
         * @return void
         */
        public static function addShortcodeEditorButton()
        {
            if ( ! current_user_can('edit_posts') && ! current_user_can('edit_pages') ) {
                return;
            }
            if ( get_user_option('rich_editing') == 'true') {
                add_filter( 'mce_external_plugins', array( __CLASS__, 'addTinymcePlugin' ));
                add_filter( 'mce_buttons', array( __CLASS__, 'registerTinymceButton' ));
            }
        }

        //}}
        //{{ addTinymcePlugin

        /**
         * Add tinyMce plugin
         *
         * @return void
         */
        public static function addTinymcePlugin( $plugin_array )
        {
            $plugin_array['ipcozr_shortcode'] = IPCOZR_BASE_URL . '/assets/js/editor-plugin.js';
            return $plugin_array;
        }

        //}}
        //{{ registerTinymceButton

        /**
         * Add tinyMce button
         *
         * @return void
         */
        public static function registerTinymceButton( $buttons )
        {
            array_push($buttons, "|", "ipcozr_shortcode");
            return $buttons;
        }

        //}}
        //{{ shortcodeEditorFilter

        /**
         * The editor filter
         *
         * @param  string $content
         * @return string
         */
        public static function shortcodeEditorFilter( $content )
        {
            global $ipcozr_useEditor;

            $ipcozr_useEditor = true;
            return $content;
        }

        //}}
        //{{ addShortcodeForm

        /**
         * Add shortcode form
         *
         * @return void
         */
        public static function addShortcodeForm()
        {
            global $ipcozr_useEditor;

            wp_enqueue_script('wp-color-picker');
            wp_enqueue_style('wp-color-picker');

            if ( $ipcozr_useEditor ) {
                $viewer = new IPCozr_Viewer();
                $viewer->shortcodeBackend();
            }
        }

        //}}
        //{{ shortcode

        /**
         * Get shortcode
         *
         * @param  array  $atts
         * @param  string $content
         * @return void
         */
        public static function shortcode( $atts, $content = '')
        {
            $viewer = new IPCozr_Viewer();
            return $viewer->shortcode( $atts, $content );
        }

        //}}
        //{{ widgetPage

        /**
         * Get widget page
         *
         * @return void
         */
        public static function widgetPage()
        {
            $viewer = new IPCozr_Viewer();
            return $viewer->widgetPage();
        }

        //}}
        //{{ addMetaBox

        /**
         * Add administrator meta box
         *
         * @return void
         */
        public static function addMetaBox()
        {
            $screens = array( 'post', 'page' );
            foreach ($screens as $screen)
            {
                add_meta_box(
                    'ipcozr_metabox',
                    __( 'Campaign Widget Settings', IPCOZR_SLUG ),
                    array( __CLASS__, 'metaBoxContent' ),
                    $screen
                );
            }
        }

        //}}
        //{{ metaBoxContent

        /**
         * Content for administrator meta box
         *
         * @return void
         */
        public static function metaBoxContent( $post )
        {
            $viewer = new IPCozr_Viewer();
            $viewer->metabox( $post );
        }

        //}}
        //{{ savePost

        /**
         * Save page/post meta box setting
         *
         * @return void
         */
        public static function savePost( $post_id )
        {
            if ( isset( $_POST['post_type'] ) && 'page' == $_POST['post_type'] ) {
                if ( ! current_user_can( 'edit_page', $post_id ) ) {
                    return;
                }
            } else {
                if ( ! current_user_can( 'edit_post', $post_id ) ) {
                    return;
                }
            }
            // Check if the user intended to change this value.
            if ( ! isset( $_POST['ipcozr_mbox_nonce'] ) ) {
                return;
            }
            $post_ID  = $_POST['post_ID'];
            //Widget
            $widgetId = sanitize_text_field( $_POST['ipcozr_widget'] );
            update_post_meta( $post_ID, '__ipcozr_page_widget', $widgetId );
        }

        //}}
        //{{ widgetInit

        /**
         * Widget innit
         */
        public function widgetInit()
        {
            require_once( IPCOZR_LIB . 'widget.php' );
            register_widget( 'IPCozr_Widget' );
        }

        //}}
        //{{ adminStyle

        /**
         * Add Css file
         *
         * @return void
         */
        public static function adminStyle()
        {
            wp_deregister_style( 'ipcozr-admin-style' );
            wp_register_style('ipcozr-admin-style', IPCOZR_BASE_URL . 'assets/css/admin.min.css', false, null);
            wp_enqueue_style('ipcozr-admin-style');

            wp_deregister_style( 'ipcozr-datetime-style' );
            wp_register_style('ipcozr-datetime-style', IPCOZR_BASE_URL . 'assets/css/jquery.datetimepicker.css', false, null);
            wp_enqueue_style('ipcozr-datetime-style');
        }

        //}}
        //{{ adminGlobalStyle

        /**
         * Add Css file to all pages
         *
         * @return void
         */
        public static function adminGlobalStyle()
        {
            wp_deregister_style( 'ipcozr-admin-global-style' );
            wp_register_style('ipcozr-admin-global-style', IPCOZR_BASE_URL . 'assets/css/admin-global.min.css', false, null);
            wp_enqueue_style('ipcozr-admin-global-style');
        }

        //}}
        //{{ adminScript

        /**
         * Add javascript file
         *
         * @return void
         */
        public static function adminScript()
        {
        ?>
            <script type="text/javascript">
                var intm_img_url   = '<?php echo IPCOZR_BASE_URL . 'img/'; ?>',
                    ui_base_image  = '<?php echo IPCOZR_BASE_URL . 'assets/img/'; ?>',
                    ui_message     =
                    {
                        delete_confirm_title: '<?php _e( 'Delete Confirmation', IPCOZR_SLUG ); ?>',
                        delete_confirm_info: '<?php _e( 'Are you sure doing this action?', IPCOZR_SLUG ); ?>'
                    }
            </script>
        <?php
            wp_deregister_script('ipcozr-admin-script');
            wp_enqueue_script(
                'ipcozr-admin-script',
                IPCOZR_BASE_URL . 'assets/js/admin.min.js',
                array( 'jquery' ),
                1,
                true
            );
        }

        //}}
        //{{ adminGlobalScript

        /**
         * Add javascript file to all pages
         *
         * @return void
         */
        public static function adminGlobalScript()
        {
            wp_deregister_script('ipcozr-admin-global-script');
            wp_enqueue_script(
                'ipcozr-admin-global-script',
                IPCOZR_BASE_URL . 'assets/js/admin-global.min.js',
                array( 'jquery' ),
                1,
                true
            );
        }

        //}}
        //{{ publicStyle

        /**
         * Print public style
         *
         * @return void
         */
        public static function publicStyle()
        {
        ?>
            <script type="text/javascript">
                var ipcozr_ajaxurl = "<?php echo admin_url( 'admin-ajax.php' ); ?>";
            </script>
            <link media="all" type="text/css" href="<?php echo IPCOZR_BASE_URL; ?>assets/css/public.min.css" rel="stylesheet">
        <?php
        }

        //}}
        //{{ publicScript

        /**
         * Print public script
         *
         * @return void
         */
        public static function publicScript()
        {
            wp_enqueue_script('jquery');

            wp_deregister_script('jquery-ui');
            wp_register_script('jquery-ui', "http" . ($_SERVER['SERVER_PORT'] == 443 ? "s" : "") . "://ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/jquery-ui.min.js", false, null);
            wp_enqueue_script('jquery-ui');

            wp_deregister_script('ipcozr-public-script');
            wp_enqueue_script(
                'ipcozr-public-script',
                IPCOZR_BASE_URL . 'assets/js/public.min.js',
                array( 'jquery' ),
                1,
                true
            );
        }

        //}}
    }
}
