<?php
/**
 * Controller to handling updating campaign process
 *
 * @package wp-email-countdown
 * @author InstaProduct
 * @link http://insta-product.com/
 */

if ( ! class_exists( 'IPCozr_Ajax_Update' ))
{
    class IPCozr_Ajax_Update
    {
        /**
         * Execution save options process
         *
         * @return void
         */
        public function exec()
        {
            $redirect = false;
            $is_add   = false;
            $error    = true;
            if ( ! empty( $_POST['isadd'] )) {
                $is_add = true;
            }
            if ( ! empty( $_POST['campaign_id'] )) {
                $campaignId = absint( $_POST['campaign_id'] );
                if ( $campaign = get_post( $campaignId ) ) {
                    $this->_updateCampaign( $campaign );
                } else {
                    $campaignId = 0;
                }
            } else {
                $campaignId = $this->_addCampaign();
            }
            if ( $campaignId )
            {
                $error = false;
                $model = new IPCozr_Model();
                $model->saveSetting( $campaignId );
            }
            if ( $error ) {
                $status   = 0;
                $campaignId  = 0;
                if ( $is_add ) {
                    $message = __( 'Failed create campaign.', IPCOZR_SLUG );
                } else {
                    $message = __( 'Failed save campaign.', IPCOZR_SLUG );
                }
            } else {
                $status = 1;
                if ( $is_add ) {
                    $message  = __( 'Successfully created campaign.', IPCOZR_SLUG );
                    $redirect = admin_url() . 'admin.php?page=' . IPCOZR_PAGENAME . '&action=edit&id=' . $campaignId . '&create=1&tab=ipcozr-section-countdown';
                } else {
                    $message = __( 'Successfully saved campaign.', IPCOZR_SLUG );
                }
            }
            $result = array(
                'status'    => $status,
                'message'   => $message,
                'campaignid'   => $campaignId
            );
            if ( $redirect ) {
                $result['redirect'] = $redirect;
            }
            echo json_encode( $result );
            die();
        }

        /**
         * Add new campaign
         *
         * @return int
         */
        protected function _addCampaign()
        {
            global $user_ID;

            if ( empty( $_POST['title'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            $dataPost = array(
                'post_status'  => $status,
                'post_type'    => 'ipcozr-item',
                'post_author'  => $user_ID,
                'post_title'   => $title,
                'post_content' => ''
            );
            $result = wp_insert_post( $dataPost );
            return $result;
        }

        /**
         * Update campaign
         *
         * @param  object $campaign
         * @return boolean
         */
        protected function _updateCampaign( $campaign )
        {
            $result   = 0;
            $dataPost = array();
            if ( ! empty( $_POST['title'] ) && $_POST['title'] != $campaign->post_title ) {
                $dataPost['post_title'] = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            if ( $status != $campaign->post_status ) {
                $dataPost['post_status'] = $status;
            }
            if ( ! empty( $dataPost )) {
                $dataPost['ID'] = $campaign->ID;
                $result = wp_update_post( $dataPost );
            }
            return $result;
        }
    }
}
