<?php
/**
 * Controller to handling updating promo process
 *
 * @package wp-email-countdown
 * @author InstaProduct
 * @link http://insta-product.com/
 */

if ( ! class_exists( 'IPCozr_Ajax_Update_promo' ))
{
    class IPCozr_Ajax_Update_promo
    {
        /**
         * Execution save options process
         *
         * @return void
         */
        public function exec()
        {
            $redirect = false;
            $is_add   = false;
            $error    = true;
            if ( ! empty( $_POST['isadd'] )) {
                $is_add = true;
            }
            if ( ! empty( $_POST['promo_id'] )) {
                $promoId = absint( $_POST['promo_id'] );
                if ( $promo = get_post( $promoId ) ) {
                    $this->_updatePromo( $promo );
                } else {
                    $promoId = 0;
                }
            } else {
                $promoId = $this->_addPromo();
            }
            if ( $promoId )
            {
                $error = false;
                $model = new IPCozr_Model_Promo();
                $model->saveSetting( $promoId );
            }
            if ( $error ) {
                $status   = 0;
                $promoId  = 0;
                if ( $is_add ) {
                    $message = __( 'Failed create promo page.', IPCOZR_SLUG );
                } else {
                    $message = __( 'Failed save promo page.', IPCOZR_SLUG );
                }
            } else {
                $status = 1;
                if ( $is_add ) {
                    $message  = __( 'Successfully created promo page.', IPCOZR_SLUG );
                    $redirect = admin_url() . 'admin.php?page=' . IPCOZR_PAGENAME . '-promo&action=edit&id=' . $promoId . '&create=1&tab=ipcozr-section-promo-content';
                } else {
                    $message = __( 'Successfully saved promo page.', IPCOZR_SLUG );
                }
            }
            $result = array(
                'status'    => $status,
                'message'   => $message,
                'promoid'   => $promoId
            );
            if ( $redirect ) {
                $result['redirect'] = $redirect;
            }
            echo json_encode( $result );
            die();
        }

        /**
         * Add new promo
         *
         * @return int
         */
        protected function _addPromo()
        {
            global $user_ID;

            //Title
            if ( empty( $_POST['title'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['title'] );
            }

            //Slug
            if ( ! empty( $_POST['slug'] )) {
                $slug  = apply_filters( 'pre_term_name', $_POST['slug'] );
            } else {
                $slug = sanitize_title( $_POST['title'] );
                $slug = wp_unique_post_slug( $slug, 0, 1, 'ipcozr-item', 0 );
            }

            //Status
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            $dataPost = array(
                'post_status'  => $status,
                'post_type'    => 'ipcozr-promo',
                'post_author'  => $user_ID,
                'post_title'   => $title,
                'post_name'    => $slug,
                'post_content' => ''
            );
            $result = wp_insert_post( $dataPost );
            return $result;
        }

        /**
         * Update promo
         *
         * @param  object $promo
         * @return boolean
         */
        protected function _updatePromo( $promo )
        {
            $result   = 0;
            $dataPost = array();
            if ( ! empty( $_POST['title'] ) && $_POST['title'] != $promo->post_title ) {
                $dataPost['post_title'] = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            if ( $status != $promo->post_status ) {
                $dataPost['post_status'] = $status;
            }
            if ( ! empty( $dataPost )) {
                $dataPost['ID'] = $promo->ID;
                $result = wp_update_post( $dataPost );
            }
            return $result;
        }
    }
}
