<?php
if ( ! class_exists( 'IPCozr_Countdown' )) {

    /**
     * Class countdown creator
     */
    class IPCozr_Countdown
    {
        //{{ parameters

        /**
         * Take campaign object
         *
         * @var object
         */
        protected $_campaign;

        /**
         * Take campaign setting
         *
         * @var object
         */
        protected $_setting;

        //}}
        //{{ __construct

        /**
         * Class constructor
         *
         * @param  object $campaign
         * @return void
         */
        public function __construct( $campaign )
        {
            $this->_campaign = $campaign;
            $this->_setting  = new IPCozr_Setting( $campaign->ID, '__ipcozr-setting' );
        }

        //}}
        //{{ render

        /**
         * Render countdown
         *
         * @return string
         */
        public function render()
        {
            global $wp_query;

            $query    = $wp_query->query;
            $setting  = $this->_setting;
            $timezone = $setting->getSetting( 'timezone' );

            date_default_timezone_set( $timezone );
            require_once( IPCOZR_LIB . '/GIFEncoder.class.php' );

            $endDate    = $setting->getSetting( 'end_date' );
            $timeFuture = strtotime( $endDate );
            $timeNow    = time();

            if ( $timeNow < $timeFuture )
            {
                //
                //Save stat
                if ( ! isset( $query['campaign_preview'] ) || empty( $query['campaign_preview'] )) {
                    $this->_saveStat();
                }
                //
                //Render
                $this->_renderCountdown( $timeNow, $timeFuture );
            }
            else {
                $this->_renderEnded();
            }
        }

        //}}
        //{{ _saveStat

        /**
         * Save stat
         *
         * @return string
         */
        public function _saveStat()
        {
            $campaign = $this->_campaign;
            $model    = new IPCozr_Model_Stat();
            //
            //Save display count
            $model->saveStat( $campaign, 'display' );
            //
            //Save unique display count
            $access = $model->getUniqueAccess( 1, $campaign );
            if ( empty( $access )) {
                $model->saveStat( $campaign, 'unique_display' );
            }
            //
            //Save access
            $model->insertAccess( 1, $campaign );
        }

        //}}
        //{{ _renderEnded

        /**
         * Render countdown
         *
         * @return string
         */
        protected function _renderEnded()
        {
            if ( $imageId = $this->_setting->getSetting( 'end_image' ) )
            {
                $imageFile = get_attached_file( $imageId );
                if ( file_exists( $imageFile ))
                {
                    $imageInfo = getimagesize( $imageFile );
                    $mime      = $imageInfo['mime'];
                    $image     = file_get_contents( $imageFile );

                    header( 'Content-type:' . $mime );
                    echo $image;
                }
            }
        }

        //}}
        //{{ _renderCountdown

        /**
         * Render countdown
         *
         * @return string
         */
        protected function _renderCountdown( $timeNow, $timeFuture )
        {
            $now    = new DateTime( date( 'r', $timeNow ));
            $future = new DateTime( date( 'r', $timeFuture ));

            $frames = array();
            $delays = array();

            $imagePath = $this->_getBaseImage();
            $image  = imagecreatefrompng( $imagePath );
            $delay  = 100; // milliseconds
            $font   = array(
              'size'      => $this->_getFontSize(),
              'angle'     => 0,
              'x-offset'  => $this->_getFontX(),
              'y-offset'  => $this->_getFontY(),
              'file'      => $this->_getFont(),
              'color'     => imagecolorallocate($image, 255, 255, 255),
            );

            for( $i = 0; $i <= 60; $i++ )
            {
                $interval = date_diff( $future, $now );
                if ( $future < $now )
                {
                    // Open the first source image and add the text.
                    $imagePath = $this->_getBaseImage();
                    $image     = imagecreatefrompng( $imagePath );
                    $text      = $interval->format( '00:00:00:00' );

                    imagettftext(
                      $image,
                      $font['size'],
                      $font['angle'],
                      $font['x-offset'],
                      $font['y-offset'],
                      $font['color'],
                      $font['file'],
                      $text
                    );
                    ob_start();
                    imagegif ($image );
                    $frames[] = ob_get_contents();
                    $delays[] = $delay;
                    $loops    = 1;
                    ob_end_clean();
                    break;
                }
                else
                {
                    $text = $interval->format( '%a:%H:%I:%S' );
                    // %a is weird in that it doesn’t give you a two digit number
                    // check if it starts with a single digit 0-9
                    // and prepend a 0 if it does
                    if( preg_match( '/^[0-9]\:/', $text ) ){
                        $text = '0' . $text;
                    }
                    $textTemp = $text;
                    $parts    = explode( ':', $textTemp );
                    $first    = array_shift( $parts );
                    if ( strlen( $first ) == 3 ) {
                        $imagePath = $this->_getBaseImage( '_lg' );
                    } else {
                        $imagePath = $this->_getBaseImage();
                    }

                    // Open the first source image and add the text.
                    $image = imagecreatefrompng( $imagePath );

                    imagettftext(
                      $image,
                      $font['size'],
                      $font['angle'],
                      $font['x-offset'],
                      $font['y-offset'],
                      $font['color'],
                      $font['file'],
                      $text
                    );

                    ob_start();
                    imagegif( $image );
                    $frames[] = ob_get_contents();
                    $delays[] = $delay;
                    $loops    = 0;
                    ob_end_clean();
                }
                $now->modify( '+1 second' );
            }

            //expire this image instantly
            header( 'Expires: Sat, 26 Jul 1997 05:00:00 GMT' );
            header( 'Last-Modified: ' . gmdate( 'D, d M Y H:i:s' ) . ' GMT' );
            header( 'Cache-Control: no-store, no-cache, must-revalidate' );
            header( 'Cache-Control: post-check=0, pre-check=0', false );
            header( 'Pragma: no-cache' );
            $gif = new AnimatedGif( $frames, $delays, $loops );
            $gif->display();
        }

        //}}
        //{{ renderCountdown

        /**
         * Get base image path
         *
         * @return string
         */
        public function _getBaseImage( $suffix = '' )
        {
            $template   = $this->_setting->getSetting( 'template', 'default' );
            $style      = $this->_setting->getSetting( 'style', 'black' );

            $imageName  = 'countdown_' . $template . '_' . $style;
            if ( ! empty( $suffix )) {
                $imageName .= $suffix;
            }
            $imageName .= '.png';
            $imageFile  = IPCOZR_PATH . 'assets/img/template/' . $imageName;
            return $imageFile;
        }

        //}}
        //{{ _getFont

        /**
         * Get font path
         *
         * @return string
         */
        public function _getFont()
        {
            $font = $this->_setting->getSetting( 'font', 'DigitalDream' );
            $fontFile = IPCOZR_PATH . 'assets/fonts/' . $font . '.ttf';
            return $fontFile;
        }

        //}}
        //{{ _getFontSize

        /**
         * Get font size
         *
         * @return string
         */
        public function _getFontSize()
        {
            $font = $this->_setting->getSetting( 'font', 'DigitalDream' );
            if ( $font == 'AnonymousPro' ) {
                $fontSize = 55;
            } elseif ( $font == 'Cousine' ) {
                $fontSize = 52;
            } elseif ( $font == 'CutiveMono' ) {
                $fontSize = 52;
            } elseif ( $font == 'FiraMono' ) {
                $fontSize = 52;
            } elseif ( $font == 'Inconsolata' ) {
                $fontSize = 60;
            } elseif ( $font == 'NovaMono' ) {
                $fontSize = 55;
            } elseif ( $font == 'OxygenMono' ) {
                $fontSize = 52;
            } elseif ( $font == 'PTMono' ) {
                $fontSize = 52;
            } elseif ( $font == 'RobotoMono' ) {
                $fontSize = 52;
            } elseif ( $font == 'ShareTechMono' ) {
                $fontSize = 56;
            } elseif ( $font == 'SourceCodePro' ) {
                $fontSize = 54;
            } elseif ( $font == 'UbuntuMono' ) {
                $fontSize = 62;
            } else {
                $fontSize = 40;
            }
            return $fontSize;
        }

        //}}
        //{{ _getFontX

        /**
         * Get font x position
         *
         * @return string
         */
        public function _getFontX()
        {
            $font = $this->_setting->getSetting( 'font', 'DigitalDream' );
            if ( $font == 'AnonymousPro' ) {
                $fontX = 32;
            } elseif ( $font == 'Cousine' ) {
                $fontX = 24;
            } elseif ( $font == 'CutiveMono' ) {
                $fontX = 15;
            } elseif ( $font == 'FiraMono' ) {
                $fontX = 22;
            } elseif ( $font == 'Inconsolata' ) {
                $fontX = 30;
            } elseif ( $font == 'NovaMono' ) {
                $fontX = 22;
            } elseif ( $font == 'OxygenMono' ) {
                $fontX = 22;
            } elseif ( $font == 'PTMono' ) {
                $fontX = 22;
            } elseif ( $font == 'RobotoMono' ) {
                $fontX = 22;
            } elseif ( $font == 'ShareTechMono' ) {
                $fontX = 24;
            } elseif ( $font == 'SourceCodePro' ) {
                $fontX = 12;
            } elseif ( $font == 'UbuntuMono' ) {
                $fontX = 16;
            } else {
                $fontX = 10;
            }
            return $fontX;
        }

        //}}
        //{{ _getFontY

        /**
         * Get font Y position
         *
         * @return string
         */
        public function _getFontY()
        {
            $font     = $this->_setting->getSetting( 'font', 'DigitalDream' );
            $template = $this->_setting->getSetting( 'template', 'default' );
            if ( $template == 'default_top' ) {
                $fontY = 90;
            } elseif ( $template == 'framed_top' ) {
                $fontY = 96;
            } elseif ( $template == 'gradation_top' ) {
                $fontY = 96;
            } elseif ( $template == 'stripes_top' ) {
                $fontY = 96;
            } elseif ( $template == 'floral' ) {
                if ( $font == 'DigitalDream' ) {
                  $fontY = 62;
                } else {
                  $fontY = 70;
                }
            } elseif ( $template == 'floral_top' ) {
                $fontY = 96;
            } elseif ( $template == 'grungy' ) {
                if ( $font == 'DigitalDream' ) {
                  $fontY = 62;
                } else {
                  $fontY = 70;
                }
            } elseif ( $template == 'grungy_top' ) {
                $fontY = 96;
            } elseif ( $template == 'retro' ) {
                if ( $font == 'DigitalDream' ) {
                  $fontY = 62;
                } else {
                  $fontY = 70;
                }
            } elseif ( $template == 'retro_top' ) {
                $fontY = 96;
            } else {
                $fontY = 70;
            }
            return $fontY;
        }
    }
}
