<?php
if ( ! class_exists( 'IPCozr_Db' )) {

    /**
     * Class for manage plugin database
     */
    class IPCozr_Db
    {
        /**
         * Take db version
         *
         * @var string
         */
        public $version = '1.0';

        /**
         * Class construct
         *
         * @return void
         */
        public function __construct()
        {
            global $ipcozr_db_version;
            $ipcozr_db_version = $this->version;
        }

        /**
         * Database query
         *
         * @return string
         */
        public function query_stats()
        {
            global $wpdb;

            $table_name = $wpdb->prefix . 'ipcozr_stats';
            $sql = "CREATE TABLE $table_name (
                        stat_id             int(11)             NOT NULL AUTO_INCREMENT,
                        campaign_id         int(11)             NOT NULL,
                        stat_date           datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        stat_display        int(11)             NOT NULL,
                        stat_unique_display int(11)             NOT NULL,
                        stat_click          int(11)             NOT NULL,
                        stat_unique_click   int(11)             NOT NULL,
                    PRIMARY KEY stat_id (stat_id),
                    KEY campaign_id (campaign_id)
            );";

            return $sql;
        }

        /**
         * Database query
         *
         * @return string
         */
        public function query_access_detail()
        {
            global $wpdb;

            $table_name = $wpdb->prefix . 'ipcozr_access_detail';
            $sql = "CREATE TABLE $table_name (
                        access_id          int(11)             NOT NULL AUTO_INCREMENT,
                        campaign_id        int(11)             NOT NULL,
                        access_date        datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        access_ip          varchar(25)         NOT NULL,
                        access_user_agent  varchar(255)        NOT NULL,
                        access_type        smallint            NOT NULL,
                    PRIMARY KEY access_id (access_id),
                    KEY campaign_id (campaign_id)
            );";

            return $sql;
        }

        /**
         * Database query
         *
         * @return string
         */
        public function query()
        {
            $sql  = '';
            $sql .= $this->query_stats();
            $sql .= $this->query_access_detail();
            return $sql;
        }

        /**
         * install
         *
         * @return void
         */
        public function install()
        {
            global $wpdb, $ipcozr_db_version;
            $installed_ver = get_option( "ipcozr_db_version" );

            if( ! $installed_ver || ( $installed_ver != $ipcozr_db_version ) ) {

                $sql = $this->query();

                require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
                dbDelta( $sql );

                update_option( "ipcozr_db_version", $ipcozr_db_version );
            }
        }

        /**
         * update
         *
         * @return void
         */
        public function update()
        {
            global $ipcozr_db_version;
            if ( get_site_option( 'ipcozr_db_version' ) != $ipcozr_db_version ) {
                $this->install();
            }
        }
    }

}
?>
