<?php
if ( ! class_exists( 'IPCozr_Table_Main' ))
{
    /**
     * Class for create list object
     */
    class IPCozr_Table_Main
    {
        /**
         * Take query object
         * @var WP_Query
         */
        protected $_query;

        /**
         * Take data result
         * @var int
         */
        protected $_result;

        /**
         * Take number of data
         * @var int
         */
        protected $_total;

        /**
         * Take number of page
         * @var int
         */
        protected $_total_page;

        /**
         * Take page number
         * @var int
         */
        protected $_page;

        /**
         * Take number of data per page
         * @var int
         */
        protected $_per_page;

        /**
         * Class constructor
         *
         * @return void
         */
        public function __construct()
        {

            $page = 1;
            if ( ! empty( $_GET['paged'] )) {
                $page = absint( $_GET['paged'] );
            }
            $this->_page     = $page;
            $this->_per_page = 12;
        }

        /**
         * Create list
         *
         * @return string
         */
        public function create()
        {
            //Get data
            $this->_get_data();
            //Build list
            return $this->_build();
        }

        /**
         * Render list
         *
         * @return void
         */
        public function render()
        {
            echo $this->create();
        }

        /**
         * Get data
         *
         * @return void
         */
        protected function _get_data()
        {
            $page     = $this->_page;
            $per_page = $this->_per_page;
            $args = array(
                'orderby'        => 'post_date',
                'order'          => 'DESC',
                'post_type'      => 'ipcozr-item',
                'post_status'    => array('publish','draft'),
                'posts_per_page' => $per_page,
                'paged'          => $page
            );
            $query = new WP_Query;
            $this->_result = $query->query( $args );
            $this->_query  = $query;
            $this->_total  = $query->found_posts;
            $this->_total_page = $this->_get_total_page();
            return $this->_result;
        }

        /**
         * Build list
         *
         * @return void
         */
        protected function _build()
        {
            $list  = $this->_get_action_top();
            $list .= $this->_get_list();
            $list .= $this->_get_action_bottom();
            $list  = $this->_get_container( $list );
            return $list;
        }

        /**
         * Get list
         *
         * @return string
         */
        protected function _get_list()
        {
            $list = '<div class="ipcozr-the-list">';
            if ( empty( $this->_result )) {
                $list .= $this->_get_empty_list();
            }
            else
            {
                $index = 1;
                foreach( $this->_result as $item ) {
                    $list .= $this->_get_list_item( $item, $index );
                    $index++;
                }
            }
            $list .= '</div>';
            return $list;
        }

        /**
         * Get list item
         *
         * @param  object $item
         * @param  int    $index
         * @return string
         */
        protected function _get_list_item( $item, $index )
        {
            $alt = '';
            if ( $index % 2 ) {
                $alt = ' ipcozr-alt';
            }
            $result  = '<div id="ipcozr-item-' . $item->ID . '" class="ipcozr-list-item' . $alt . '">';
            $result .= '<div class="ipcozr-item-inner row">';
            $result .= $this->_get_item_main  ( $item, $index );
            $result .= $this->_get_item_action( $item, $index );
            $result .= '</div>';
            $result .= '</div>';
            return $result;
        }

        /**
         * Get item main
         *
         * @param  object $item
         * @param  int    $index
         * @return string
         */
        protected function _get_item_main( $item, $index )
        {
            $title  = $item->post_title;
            $date   = mysql2date( 'F d Y, h:i', $item->post_date );
            $link   = $this->_get_link_edit( $item );
            if ( $item->post_status == 'publish' ) {
                $label = '<span class="ipcozr-label ipcozr-label-success">' . __( 'Publish', IPCOZR_SLUG ) . '</span>';
            } else {
                $label = '<span class="ipcozr-label ipcozr-label-danger">' . __( 'Draft', IPCOZR_SLUG ) . '</span>';
            }
            $setting = new IPCozr_Setting( $item->ID, '__ipcozr-setting' );
            $result = '<div class="col-sm-6">' .
                            '<div class="ipcozr-clear">' .
                                '<div class="ipcozr-item-main">' .
                                    '<h3 class="ipcozr-item-title">' .
                                        '<a href="' . $link . '">' . $title . '</a>' .
                                        $label .
                                    '</h3>' .
                                    '<p class="ipcozr-item-create">' .
                                        '<i class="ipcozr-fa ipcozr-fa-calendar"></i>' .
                                        $date .
                                    '</p>' .
                                '</div>' .
                            '</div>' .
                       '</div>';
            return $result;
        }

        /**
         * Get item action
         *
         * @param  object $item
         * @param  int    $index
         * @return string
         */
        protected function _get_item_action( $item, $index )
        {
            $result  = '<div class="col-sm-6">' .
                            '<div class="ipcozr-item-action ipcozr-clear text-right">' .
                                $this->_getItemStat( $item, $index ) .

                                '<div class="ipcozr-item-nav ipcozr-fademenu">' .
                                '<div class="ipcozr-fademenu-inner">' .
                                    '<div class="ipcozr-nav-label ipcozr-fademenu-button">' .
                                        '<a class="ipcozr-menu-link" href="#"><i class="ipcozr-fa ipcozr-fa-bars"></i></a>' .
                                    '</div>' .
                                    '<ul class="ipcozr-submenu ipcozr-fademenu-content">' .
                                        '<li>' .
                                            '<a href="' . $this->_get_link_edit( $item ) . '" class="ipcozr-btn ipcozr-btn-xs ipcozr-btn-success">' .
                                                '<i class="ipcozr-fa ipcozr-fa-pencil"></i>' .
                                                 __( 'Edit Campaign', IPCOZR_SLUG ) .
                                            '</a>' .
                                        '</li>' .
                                        '<li>' .
                                            '<a href="#" class="ipcozr-btn ipcozr-btn-xs ipcozr-btn-danger ipcozr-item-delete" data-id="' . $item->ID . '">' .
                                                '<i class="ipcozr-fa ipcozr-fa-trash"></i>' .
                                                __( 'Delete Campaign', IPCOZR_SLUG ) .
                                            '</a>' .
                                        '</li>' .
                                    '</ul>' .
                                '</div>' .
                                '</div>' .
                            '</div>' .
                       '</div>';
            return $result;
        }

        /**
         * Get item action
         *
         * @param  object $item
         * @param  int    $index
         * @return string
         */
        protected function _getItemStat( $item, $index )
        {
            $model    = new IPCozr_Model_Stat();
            $display  = $model->getStatValue( $item->ID, 'display' );
            $clicks   = $model->getStatValue( $item->ID, 'click' );
            $result  = '<div class="ipcozr-item-stats">' .
                            '<div class="ipcozr-clear">' .
                                '<div class="ipcozr-stat ipcozr-stat-view">' .
                                    '<div class="ipcozr-stat-value">' .
                                        '<strong>' . $model->restyleText( $display ) . '</strong>' .
                                    '</div>' .
                                    '<div class="ipcozr-stat-label">' .
                                        '<i class="ipcozr-fa ipcozr-fa-eye"></i>' .
                                        '<em>' . __( 'Display', IPCOZR_SLUG ) . '</em>' .
                                    '</div>' .
                                '</div>' .
                                '<div class="ipcozr-stat ipcozr-stat-click">' .
                                    '<div class="ipcozr-stat-value">' .
                                        '<strong>' . $model->restyleText( $clicks ) . '</strong>' .
                                    '</div>' .
                                    '<div class="ipcozr-stat-label">' .
                                        '<i class="ipcozr-fa ipcozr-fa-check"></i>' .
                                        '<em>' . __( 'Clicks', IPCOZR_SLUG ) . '</em>' .
                                    '</div>' .
                                '</div>' .
                            '</div>' .
                       '</div>';

            return $result;
        }

        /**
         * Get link edit
         *
         * @param  object $item
         * @return string
         */
        protected function _get_link_edit( $item )
        {
            $link = admin_url() . 'admin.php?page=' . IPCOZR_PAGENAME .  '&action=edit&id=' . $item->ID;
            return $link;
        }

        /**
         * Get empty list
         *
         * @return string
         */
        protected function _get_empty_list()
        {
            return '<div class="ipcozr-list-empty">' .
                        '<p><em>' . __( 'Data not available', IPCOZR_SLUG ) . '</em></p>' .
                    '</div>';
        }

        /**
         * Get pagination
         *
         * @return string
         */
        protected function _get_pagination()
        {
            $result = '';
            if ( $this->_need_pagination() )
            {
                $result = '<div class="ipcozr-pagination ipcozr-clear">' .
                            $this->_get_pagination_total() .
                            $this->_get_pagination_menu() .
                          '</div>';
            }
            return $result;
        }

        /**
         * Check is need pagination or not
         *
         * @return string
         */
        protected function _need_pagination()
        {
            $is_need = false;
            if ( ! empty( $this->_result ) && $this->_total_page > 1 ) {
                $is_need = true;
            }
            return $is_need;
        }

        /**
         * Get total page
         *
         * @return int
         */
        protected function _get_total_page()
        {
            $total_page = ceil( $this->_total/$this->_per_page );
            return $total_page;
        }

        /**
         * Get pagination total info
         *
         * @return string
         */
        protected function _get_pagination_total()
        {
            $total  = $this->_total;
            $result = '<div class="ipcozr-total">' .
                        sprintf( _n( '1 item', '%s items', $total, IPCOZR_SLUG ), $total ) .
                      '</div>';
            return $result;
        }

        /**
         * Get pagination menu
         *
         * @return string
         */
        protected function _get_pagination_menu()
        {
            $result = '<div class="ipcozr-pagination-menu">' .
                        $this->_get_pagination_first() .
                        $this->_get_pagination_previous() .
                        $this->_get_pagination_page_info() .
                        $this->_get_pagination_next() .
                        $this->_get_pagination_last() .
                      '</div>';
            return $result;
        }

        /**
         * Get pagination go first menu
         *
         * @return string
         */
        protected function _get_pagination_first()
        {
            $result = '';
            $link   = $this->_get_pagination_link( 1 );
            $class  = 'ipcozr-pagin ipcozr-pagin-first';
            if ( $this->_page == 1 || $this->_total_page < 3 ) {
                $class .= ' disabled';
            }
            $result = '<a class="' . $class . '" href="' . $link . '"><i class="ipcozr-fa ipcozr-fa-angle-double-left"></i></a>';
            return $result;
        }

        /**
         * Get pagination go last menu
         *
         * @return string
         */
        protected function _get_pagination_last()
        {
            $result = '';
            $link   = $this->_get_pagination_link( $this->_total_page );
            $class  = 'ipcozr-pagin ipcozr-pagin-last';
            if ( $this->_page == $this->_total_page || $this->_total_page < 3 ) {
                $class .= ' disabled';
            }
            $result = '<a class="' . $class . '" href="' . $link . '"><i class="ipcozr-fa ipcozr-fa-angle-double-right"></i></a>';
            return $result;
        }

        /**
         * Get pagination previous menu
         *
         * @return string
         */
        protected function _get_pagination_previous()
        {
            $result = '';
            $class  = 'ipcozr-pagin ipcozr-pagin-previous';
            $page   = $this->_page - 1;
            if ( $page < 1 ) {
                $page = 1;
                $class .= ' disabled';
            }
            $link = $this->_get_pagination_link( $page );
            $result = '<a class="' . $class . '" href="' . $link . '"><i class="ipcozr-fa ipcozr-fa-angle-left"></i></a>';
            return $result;
        }

        /**
         * Get pagination next menu
         *
         * @return string
         */
        protected function _get_pagination_next()
        {
            $result = '';
            $class  = 'ipcozr-pagin ipcozr-pagin-next';
            $page   = $this->_page + 1;
            if ( $page > $this->_total_page ) {
                $page = $this->_total_page;
                $class .= ' disabled';
            }
            $link = $this->_get_pagination_link( $page );
            $result = '<a class="' . $class . '" href="' . $link . '"><i class="ipcozr-fa ipcozr-fa-angle-right"></i></a>';
            return $result;
        }

        /**
         * Get pagination link
         *
         * @param  int $page
         * @return string
         */
        protected function _get_pagination_link( $page )
        {
            $link = admin_url() . '?page=ipcozr&paged=' . $page;
            return $link;
        }

        /**
         * Get pagination page info
         *
         * @return string
         */
        protected function _get_pagination_page_info()
        {
            $result = '<span class="ipcozr-paged-info">' .
                        '<strong>' . $this->_page . '</strong> of <strong>' . $this->_total_page . '</strong>' .
                      '</span>';
            return $result;
        }

        /**
         * Get container
         *
         * @param  string $list
         * @return string
         */
        protected function _get_container( $list )
        {
            $result = '<div class="ipcozr-manage container-fluid container">' .
                        '<div class="ipcozr-manage-list ipcozr-list-stat">' .
                            $list .
                        '</div>' .
                      '</div>';

            return $result;
        }

        /**
         * Get action top
         *
         * @return string
         */
        protected function _get_action_top()
        {
            return '';
        }

        /**
         * Get action bottom
         *
         * @return string
         */
        protected function _get_action_bottom()
        {
            $action = '<div class="ipcozr-action-bottom">' .
                        '<div class="row">' .
                            '<div class="col-sm-6"></div>' .
                            '<div class="col-sm-6 text-right">' .
                                $this->_get_pagination() .
                            '</div>' .
                        '</div>' .
                      '</div>';
            return $action;
        }
    }

}
?>
