<?php
if ( ! class_exists( 'IPCozr_Viewer' ))
{
    /**
     * Viewer class
     *
     * @package wp-email-countdown
     * @author InstaProduct
     * @link http://insta-product.com/
     */

    class IPCozr_Viewer
    {
        //{{ dashboard

        /**
         * Manage page viewer
         *
         * @return void
         */
        public function dashboard()
        {
            //Render view
            $this->pageId = IPCOZR_PAGENAME;
            $this->view( 'manage/dashboard' );
        }

        //}}
        //{{ setting

        /**
         * Manage page viewer
         *
         * @return void
         */
        public function setting()
        {
            //Render view
            $this->pageId  = IPCOZR_PAGENAME;
            $this->widgets = $this->_getWidgets();
            $this->view( 'setting' );
        }

        //}}
        //{{ addnew

        /**
         * Add new page viewer
         *
         * @return void
         */
        public function addnew()
        {
            $this->manage();
        }

        //}}
        //{{ edit

        /**
         * Edit page viewer
         *
         * @return void
         */
        public function edit()
        {
            if ( empty( $_GET['id'] )) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPCOZR_SLUG ) );
            }
            $id = absint( $_GET['id'] );
            if ( ! $campaign = get_post( $id ) ) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPCOZR_SLUG ) );
            }
            $this->manage( $campaign );
        }

        //}}
        //{{ manage

        /**
         * Manage page viewer
         *
         * @param object $campaign
         * @return void
         */
        public function manage( $campaign = false )
        {
            //
            //Including script
            wp_enqueue_media();
            wp_enqueue_script('wp-color-picker');
            wp_enqueue_style('wp-color-picker');

            $campaignId = 0;
            if ( $campaign ) {
                $campaignId = $campaign->ID;
            }
            $setting     = new IPCozr_Setting( $campaignId, '__ipcozr-setting' );
            $form_action = admin_url( 'admin-ajax.php' ) . '?doajax=1&ipcozr=1&action=ipcozr_update';

            if ( $campaign )
            {
                $pageTitle   = __( 'Edit Campaign', IPCOZR_SLUG );
                $submitLabel = __( 'Save Change', IPCOZR_SLUG );
                $submitId    = 'ipcozr-edit-campaign';
                $formId      = 'ipcozr-form-update';
                $submitIcon  = 'floppy-o';
                $pageId      = IPCOZR_PAGENAME;

            } else
            {
                $pageTitle   = __( 'Add Campaign', IPCOZR_SLUG );
                $submitLabel = __( 'Add Campaign', IPCOZR_SLUG );
                $submitId    = 'ipcozr-add-campaign';
                $formId      = 'ipcozr-form-add';
                $submitIcon  = 'plus-circle';
                $pageId      = IPCOZR_PAGENAME . '-addnew';
            }
            $this->pageId      = $pageId;
            $this->pageTitle   = $pageTitle;
            $this->submitLabel = $submitLabel;
            $this->submitId    = $submitId;
            $this->formId      = $formId;
            $this->submitIcon  = $submitIcon;
            $this->setting     = $setting;
            $this->campaign    = $campaign;
            $this->campaignId  = $campaignId;

            $tab = 'ipcozr-section-general';
            if ( ! empty( $_REQUEST['tab'] )) {
                $tab = $_REQUEST['tab'];
            }
            $this->tab = $tab;

            //Render view
            $this->view( 'manage/main' );
        }

        //}}
        //{{ widget

        /**
         * Widget manage page viewer
         *
         * @return void
         */
        public function widget()
        {
            //Render view
            $this->pageId = IPCOZR_PAGENAME . '-widget';
            $this->view( 'widget/dashboard' );
        }

        //}}
        //{{ widgetCreate

        /**
         * Add new page viewer
         *
         * @return void
         */
        public function widgetCreate()
        {
            $this->widgetManage();
        }

        //}}
        //{{ widgetEdit

        /**
         * Edit page viewer
         *
         * @return void
         */
        public function widgetEdit()
        {
            if ( empty( $_GET['id'] )) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPMTUBE_SLUG ) );
            }
            $id = absint( $_GET['id'] );
            if ( ! $widget = get_post( $id ) ) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPMTUBE_SLUG ) );
            }
            $this->widgetManage( $widget );
        }

        //}}
        //{{ widgetManage

        /**
         * Manage widget viewer
         *
         * @param object $widget
         * @return void
         */
        public function widgetManage( $widget = false )
        {
            //
            //Including script
            wp_enqueue_media();
            wp_enqueue_script('wp-color-picker');
            wp_enqueue_style('wp-color-picker');

            $widgetId = 0;
            if ( $widget ) {
                $widgetId = $widget->ID;
            }
            $setting     = new IPCozr_Setting( $widgetId, '__ipcozr-widget-setting' );
            $form_action = admin_url( 'admin-ajax.php' ) . '?doajax=1&ipcozr=1&action=update_widget';

            if ( $widget )
            {
                $pageTitle   = __( 'Edit Widget', IPCOZR_SLUG );
                $submitLabel = __( 'Save Change', IPCOZR_SLUG );
                $submitId    = 'ipcozr-edit-widget';
                $formId      = 'ipcozr-form-widget-update';
                $submitIcon  = 'floppy-o';
                $pageId      = IPCOZR_PAGENAME . '-widget';

            } else
            {
                $pageTitle   = __( 'Add Widget', IPCOZR_SLUG );
                $submitLabel = __( 'Add Widget', IPCOZR_SLUG );
                $submitId    = 'ipcozr-add-widget';
                $formId      = 'ipcozr-form-widget-add';
                $submitIcon  = 'plus-circle';
                $pageId      = IPCOZR_PAGENAME . '-widget';
            }
            $this->pageId      = $pageId;
            $this->pageTitle   = $pageTitle;
            $this->submitLabel = $submitLabel;
            $this->submitId    = $submitId;
            $this->formId      = $formId;
            $this->submitIcon  = $submitIcon;
            $this->setting     = $setting;
            $this->widget      = $widget;
            $this->widgetId    = $widgetId;

            $tab = 'ipcozr-section-widget-general';
            if ( ! empty( $_REQUEST['tab'] )) {
                $tab = $_REQUEST['tab'];
            }
            $this->tab = $tab;

            //Render view
            $this->view( 'widget/main' );
        }

        //}}
        //{{ promo

        /**
         * Promo manage page viewer
         *
         * @return void
         */
        public function promo()
        {
            //Render view
            $this->pageId = IPCOZR_PAGENAME . '-promo';
            $this->view( 'promo/dashboard' );
        }


        //}}
        //{{ promoCreate

        /**
         * Add new page viewer
         *
         * @return void
         */
        public function promoCreate()
        {
            $this->promoManage();
        }

        //}}
        //{{ promoEdit

        /**
         * Edit page viewer
         *
         * @return void
         */
        public function promoEdit()
        {
            if ( empty( $_GET['id'] )) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPMTUBE_SLUG ) );
            }
            $id = absint( $_GET['id'] );
            if ( ! $promo = get_post( $id ) ) {
                wp_die( __('You attempted to edit an item that doesn&#8217;t exist. Perhaps it was deleted?', IPMTUBE_SLUG ) );
            }
            $this->promoManage( $promo );
        }

        //}}
        //{{ promoManage

        /**
         * Manage promo viewer
         *
         * @param object $promo
         * @return void
         */
        public function promoManage( $promo = false )
        {
            //
            //Including script
            wp_enqueue_media();
            wp_enqueue_script('wp-color-picker');
            wp_enqueue_style('wp-color-picker');

            $promoId = 0;
            if ( $promo ) {
                $promoId = $promo->ID;
            }
            $setting     = new IPCozr_Setting( $promoId, '__ipcozr-promo-setting' );
            $form_action = admin_url( 'admin-ajax.php' ) . '?doajax=1&ipcozr=1&action=update_promo';

            if ( $promo )
            {
                $pageTitle   = __( 'Edit Promo', IPCOZR_SLUG );
                $submitLabel = __( 'Save Change', IPCOZR_SLUG );
                $submitId    = 'ipcozr-edit-promo';
                $formId      = 'ipcozr-form-promo-update';
                $submitIcon  = 'floppy-o';
                $pageId      = IPCOZR_PAGENAME . '-promo';

            } else
            {
                $pageTitle   = __( 'Add Promo', IPCOZR_SLUG );
                $submitLabel = __( 'Add Promo', IPCOZR_SLUG );
                $submitId    = 'ipcozr-add-promo';
                $formId      = 'ipcozr-form-promo-add';
                $submitIcon  = 'plus-circle';
                $pageId      = IPCOZR_PAGENAME . '-promo';
            }
            $this->pageId      = $pageId;
            $this->pageTitle   = $pageTitle;
            $this->submitLabel = $submitLabel;
            $this->submitId    = $submitId;
            $this->formId      = $formId;
            $this->submitIcon  = $submitIcon;
            $this->setting     = $setting;
            $this->promo      = $promo;
            $this->promoId    = $promoId;

            $tab = 'ipcozr-section-promo-general';
            if ( ! empty( $_REQUEST['tab'] )) {
                $tab = $_REQUEST['tab'];
            }
            $this->tab = $tab;

            //
            //Permalink root
            $home = get_home_url();
            $path = get_option( '__ipcozr_promo_base_path', 'promo' );
            $root = $home . '/' . $path . '/';
            $this->permalink_root = $root;

            //Render view
            $this->view( 'promo/main' );
        }

        //}}
        //{{ page

        /**
         * Page viewer
         *
         * @return void
         */
        public function page()
        {
            global $wp_query, $user_ID;

            $q = $wp_query->query;
            if ( ( isset( $q['ipcozr'] ) && isset( $q['promo_name'] ) ) || isset( $q['promo_id'] ) )
            {
                $this->promoPage();
            }
            elseif ( isset( $q['ipcozr'] ) &&
                  isset( $q['ipcozr_page'] ) &&
                  $q['ipcozr_page'] == 'image' &&
                  ! empty( $q['campaign_id'] )
            ){
                $this->campaignImage();
            }
            elseif ( isset( $q['ipcozr'] ) &&
                  isset( $q['ipcozr_page'] ) &&
                  $q['ipcozr_page'] == 'campaign' &&
                  ! empty( $q['campaign_id'] )
            ){
                $this->campaignPage();
            }
        }

        //}}
        //{{ campaignImage

        /**
         * Campaign image viewer
         *
         * @return void
         */
        public function campaignImage()
        {
            global $wp_query, $user_ID;

            $campaign = $wp_query->post;
            if ( $campaign->post_status == 'publish' )
            {
                require_once( IPCOZR_LIB . '/countdown.php' );
                $countdown = new IPCozr_Countdown( $campaign );
                $countdown->render();
                exit;
            }
            else
            {
                $query->query_vars['post_type'] = 'ipcozr-item-nothing';
            }
        }

        //}}
        //{{ campaignPage

        /**
         * Campaign page viewer
         *
         * @return void
         */
        public function campaignPage()
        {
            global $wp_query, $user_ID;

            $campaign = $wp_query->post;
            if ( $campaign->post_status == 'publish' )
            {
                require_once( IPCOZR_LIB . '/campaign.php' );
                $handler = new IPCozr_Campaign( $campaign );
                $handler->save();
                exit;
            }
            else
            {
                $query->query_vars['post_type'] = 'ipcozr-item-nothing';
            }
            exit;
        }

        //}}
        //{{ promoPage

        /**
         * Widget frontend viewer
         *
         * @return void
         */
        public function promoPage()
        {
            global $wp_query, $user_ID;

            $page = $wp_query->post;

            if ( $page->post_status == 'publish' )
            {
                $setting = new IPCozr_Setting( $page->ID, '__ipcozr-promo-setting' );

                //Check restricted
                if ( empty( $user_ID ) && $setting->getSetting( 'restricted', 'true' ) == 'true' )
                {
                    $redirect  = $this->permalink;
                    $login_url = wp_login_url( $redirect );
                    wp_redirect( $login_url );
                    exit;
                }

                $this->query     = $wp_query;
                $this->setting   = $setting;
                $this->page      = $page;
                $this->cssurl    = IPCOZR_BASE_URL . 'assets/css/';
                $this->jsurl     = IPCOZR_BASE_URL . 'assets/js/';
                $this->imgurl    = IPCOZR_BASE_URL . 'assets/img/';
                $this->permalink = $this->_getPermalink( $page );

                $title = $this->setting->getSetting( 'seo_meta_title' );
                $title = trim( $title );
                if ( empty( $title )) {
                    $title = $this->page->post_title;
                }
                $this->pageTitle = stripslashes( $title );

                //Render view
                $this->view( 'front/promo' );
                exit;
            }
            else
            {
                $query->query_vars['post_type'] = 'ipcozr-promo-nothing';
            }
        }

        //}}
        //{{ shortcodeBackend

        /**
         * Shortcode Backend viewer
         *
         * @return void
         */
        public function shortcodeBackend()
        {
            $this->campaigns = $this->_getCampaigns();
            //Render view
            $this->view( 'shortcode/backend/main' );
        }

        //}}
        //{{ shortcode

        /**
         * Shortcode frontend viewer
         *
         * @param array  $atts
         * @param string $content
         * @return void
         */
        public function shortcode( $atts, $content = '' )
        {
              global $user_ID;
              if ( empty( $atts['campaign'] )) {
                  return '';
              }
              //
              //Get campaign
              $campaignId = absint( $atts['campaign'] );
              if ( ! $campaign = get_post( $campaignId )) {
                  return '';
              }
              //
              //Create link campaigne
              $result = '';
              if ( $campaign->post_status == 'publish' )
              {
                  $homeLink   = home_url();
                  $imageLink .= $homeLink . '/wp-email-countdown-image?campaign_id=' . $campaign->ID;
                  $campLink  .= $homeLink . '/wp-email-countdown-campaign?campaign_id=' . $campaign->ID;

                  $result = '<p class="ipcozr-campaign">' .
                               '<a class="ipcozr-campaign-link-' . $campaign->ID . '" href="' . $campLink . '">' .
                                '<img class="ipcozr-campaign-image-' . $campaign->ID . '" src="' . $imageLink . '" alt="">' .
                               '</a>' .
                            '</p>';
              }
              return $result;
        }

        //}}
        //{{ metabox

        /**
         * Metabox options viewer
         *
         * @return void
         */
        public function metabox( $post )
        {
            $this->post    = $post;
            $this->widgets = $this->_getWidgets();
            //Render view
            $this->view( 'metabox' );
        }

        //}}
        //{{ widgetPage

        /**
         * Widget Page
         *
         * @return void
         */
        public function widgetPage()
        {
            global $user_ID, $post;

            $object = $this->_widgetPageObject();
            if ( ! empty( $object['widget'] ) )
            {
                $widget = $object['widget'];
                $type   = $object['type'];

                $this->widget  = $widget;
                $this->type    = $type;
                $this->setting = new IPCozr_Setting( $widget->ID, '__ipcozr-widget-setting' );

                //Get campaign
                $campaign = false;
                if ( $campaignId = $this->setting->getSetting( 'campaign' ) ) {
                    $campaignId = absint( $campaignId );
                    $campaign   = get_post( $campaignId );
                }

                if ( $campaign )
                {
                    //Check restricted
                    $restricted = false;
                    if ( empty( $user_ID ) && $this->setting->getSetting( 'restricted', 'true' ) == 'true' ) {
                        $restricted = true;
                    }

                    if ( $widget->post_status == 'publish' && ! $restricted )
                    {
                        //
                        //Include view
                        $this->campaign    = $campaign;
                        $this->campSetting = new IPCozr_Setting( $campaign->ID, '__ipcozr-setting' );
                        $this->view( 'front/widget' );
                    }
                }
            }
        }
        //}}
        //{{ widgetCampaignBackend

        /**
         * Widget Backend viewer
         *
         * @param array $instance
         * @param object $wp_widget
         * @return void
         */
        public function widgetCampaignBackend( $instance, $wp_widget )
        {
            $this->campaigns = $this->_getCampaigns();
            $this->instance  = $instance;
            $this->wp_widget = $wp_widget;
            //Render view
            $this->view( 'widget/sidebar/backend' );
        }

        //}}
        //{{ widgetCampaignFrontend

        /**
         * Widget frontend viewer
         *
         * @param array $atts
         * @param array $widgetArgs
         * @return void
         */
        public function widgetCampaignFrontend( $atts, $widgetArgs = array() )
        {
              global $user_ID;

              if ( empty( $atts['campaign'] )) {
                  return '';
              }

              //
              //Get content
              $campId = absint( $atts['campaign'] );
              if ( ! $campaign = get_post( $campId )) {
                  return '';
              }
              //
              //Check post status
              if ( $campaign->post_status != 'publish' ) {
                  return '';
              }
              $this->widgetArgs = $widgetArgs;
              $this->campaign   = $campaign;
              $this->setting    = new IPCozr_Setting( $campId, '__ipcozr-setting' );

              $this->args       = $atts;

              //
              //Include view
              $this->view( 'widget/sidebar/frontend' );
        }

        //}}
        //{{ _widgetPageObject

        /**
         * Get widget Page object
         *
         * @return void
         */
        protected function _widgetPageObject()
        {
            global $post;

            $widgetId = 0;
            $widget   = false;
            $type     = 'general';
            $general  = get_option( '__ipcozr_general_widget' );
            if ( ! empty( $general )) {
                $widgetId = absint( $general );
                $type     = 'general';
            } else
            {
                if ( ( is_page() || is_single() ) && ! empty( $post )) {
                    $widgetId = get_post_meta( $post->ID, '__ipcozr_page_widget', true );
                    $type     = 'page';
                } elseif ( is_home() ) {
                    $widgetId = get_option( '__ipcozr_home_widget' );
                    $type     = 'home';
                }
            }
            if ( ! empty( $widgetId )) {
                $widget = get_post( $widgetId );
            }
            return array(
                'widget' => $widget,
                'type'   => $type
            );
        }

        //}}
        //{{ view

        /**
         * Get admin view file
         *
         * @param string $filename
         * @return void
         */
        public function view( $filename )
        {
            $path  = IPCOZR_VIEW;
            $path .= strtolower( $filename ) . '.php';
            if ( file_exists( $path )) {
                include( $path );
            }
        }

        //}}
        //{{ _getPermalink

        /**
         * Get link view
         *
         * @param  object $item
         * @return string
         */
        protected function _getPermalink( $item )
        {
            $home = get_home_url();
            $base = get_option( '__ipcozr_promo_base_path', 'promo' );

            if ( get_option( 'permalink_structure' ) )  {
                $link = $home . "/$base/" . $item->post_name;
            } else {
                $link = $home . '/?promo_id=' . $item->ID . '&ipcozr=1';
            }

            return $link;
        }

        //}}
        //{{ _getWidgets

        /**
         * Get widgets list
         *
         * @return array
         */
        protected function _getWidgets()
        {
            $args = array(
                'orderby'        => 'post_title',
                'order'          => 'DESC',
                'post_type'      => 'ipcozr-widget',
                'post_status'    => array('publish'),
                'posts_per_page' => -1,
                'paged'          => 1
            );
            $query   = new WP_Query;
            $widgets = $query->query( $args );
            return $widgets;
        }

        //}}
        //{{ _getCampaigns

        /**
         * Get campaigns list
         *
         * @return array
         */
        protected function _getCampaigns()
        {
            $args = array(
                'orderby'        => 'post_title',
                'order'          => 'DESC',
                'post_type'      => 'ipcozr-item',
                'post_status'    => array('publish'),
                'posts_per_page' => -1,
                'paged'          => 1
            );
            $query   = new WP_Query;
            $widgets = $query->query( $args );
            return $widgets;
        }

        //}}
    }
}
